<?php

namespace Drupal\openid_claveunica\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\openid_claveunica\AccountProfileCheckerInterface;
use Drupal\openid_connect\OpenIDConnectClientEntityInterface;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for complete profile after login with claveunica.
 */
class ClaveUnicaController extends ControllerBase {

  /**
   * The account profile checker service.
   *
   * @var \Drupal\openid_claveunica\AccountProfileCheckerInterface
   */
  protected $accountProfileChecker;

  /**
   * ClaveUnicaController constructor.
   *
   * @param \Drupal\openid_claveunica\AccountProfileCheckerInterface $account_profile_checker
   *   The account profile checker service.
   */
  public function __construct(AccountProfileCheckerInterface $account_profile_checker) {
    $this->accountProfileChecker = $account_profile_checker;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('openid_claveunica.account_profile_checker')
    );
  }

  /**
   * Show the complete profile form.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user object.
   * @param \Drupal\openid_connect\OpenIDConnectClientEntityInterface $client
   *   The openid client entity.
   * @param string $hash
   *   The hash to compare.
   *
   * @return array
   *   The render array.
   */
  public function completeProfile(UserInterface $user, OpenIDConnectClientEntityInterface $client, string $hash): array {
    $form_state_additions = [
      'complete_profile_hash' => $hash,
      'client_settings' => $client->get('settings') ?? [],
    ];
    return $this->entityFormBuilder()->getForm($user, 'complete_profile_claveunica', $form_state_additions);
  }

  /**
   * The title callback for complete profile page.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user object.
   * @param \Drupal\openid_connect\OpenIDConnectClientEntityInterface $client
   *   The openid client entity.
   * @param string $hash
   *   The hash to compare.
   *
   * @return string
   *   The title string.
   */
  public function completeProfileTitleCallback(UserInterface $user, OpenIDConnectClientEntityInterface $client, string $hash): string {
    return (string) $client->get('settings')['title_complete_profile'] ?? '';
  }

  /**
   * The custom access check for complete profile.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   * @param \Drupal\user\UserInterface $user
   *   The user object.
   * @param \Drupal\openid_connect\OpenIDConnectClientEntityInterface $client
   *   The openid client entity.
   * @param string $hash
   *   The hash to compare.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result object.
   */
  public function completeProfileAccess(AccountInterface $account, UserInterface $user, OpenIDConnectClientEntityInterface $client, string $hash): AccessResultInterface {
    if (!$account->isAnonymous()) {
      return AccessResult::forbidden()
        ->addCacheableDependency($user);
    }

    if ($client->getPluginId() != 'claveunica' || !$client->get('status')) {
      return AccessResult::forbidden()
        ->addCacheableDependency($client);
    }

    if (!$this->accountProfileChecker->checkHashCompleteProfile($user, $hash)) {
      return AccessResult::forbidden()
        ->addCacheableDependency($user)
        ->addCacheableDependency($client);
    }

    return AccessResult::allowed()
      ->addCacheableDependency($user)
      ->addCacheableDependency($client);
  }

}
